#include "Arduino.h"
#include "./uClock.h"
#include <LiquidCrystal.h>

// MIDI clock, start and stop byte definitions - based on MIDI 1.0 Standards.
#define MIDI_CLOCK 0xF8
#define MIDI_START 0xFA
#define MIDI_STOP  0xFC

LiquidCrystal lcd(2,3,4,5,6,7);

// Define inputs and outputs
const int button1Pin=8;   //PLAY
const int button2Pin=9;   //STOP
const int button3Pin=12;  //WRITE
const int button4Pin=11;  //F1
const int button5Pin=10;  //F2

const int TempoCVPin = A0; // =Tempo potentiometer
unsigned long Tempo;
unsigned long TempoPrev;
// Variables to handle the buttons and digital inputs
int button1State=0; 
int button2State=0; 
int button3State=0;
int button4State=0;
int button5State=0;
int button1StatePrev=0;
int button2StatePrev=0;
int button3StatePrev=0;
int button4StatePrev=0;
int button5StatePrev=0;


// The callback function wich will be called by Clock each Pulse of 96  QN clock resolution.
void ClockOut96PPQN(uint32_t * tick) 
{
  // Send MIDI_CLOCK to external gears
  Serial.write(MIDI_CLOCK);
}

// The callback function wich will be called when clock starts by using Clock.start() method.
void onClockStart() 
{
  Serial.write(MIDI_START);
}

// The callback function wich will be called when clock stops by using Clock.stop() method.
void onClockStop() 
{
  Serial.write(MIDI_STOP);
}

void setup() 
{
  lcd.begin(16, 2);

  // Initialize serial communication at 31250 bits per second, the default MIDI serial speed communication:
  Serial.begin(31250);

  // Inits the clock
  uClock.init();
  // Set the callback function for the clock output to send MIDI Sync message.
  uClock.setClock96PPQNOutput(ClockOut96PPQN);
  // Set the callback function for MIDI Start and Stop messages.
  uClock.setOnClockStartOutput(onClockStart);  
  uClock.setOnClockStopOutput(onClockStop);
  // Set the clock BPM to 126 BPM
  uClock.setTempo(120);
  TempoPrev=120;
  
  // Starts the clock, tick-tac-tick-tac...
  uClock.start();
  
  pinMode(button1Pin, INPUT);
  pinMode(button2Pin, INPUT);
  pinMode(button3Pin, INPUT);
  pinMode(button4Pin, INPUT);
  pinMode(button5Pin, INPUT);
  lcd.setCursor(0,0);
  lcd.print(F("MC-1 V1.0"));
  delay(1000);
  lcd.setCursor(0,0);
  lcd.print(F("STOP"));
}

// Do it whatever to interface with Clock.stop(), Clock.start(), Clock.setTempo() and integrate your environment...
void loop() 
{
  /* READ BUTTONS */ 
  button1State=!digitalRead(button1Pin);
  button2State=!digitalRead(button2Pin);
  button3State=!digitalRead(button3Pin);
  button4State=!digitalRead(button4Pin);
  button5State=!digitalRead(button5Pin);
  unsigned long TempoPot=((analogRead(TempoCVPin)/4) + 20 ) ;
  
  if ((button3State == 1) && (button4State == 1)) {
    Tempo=TempoPot;
  }
  
  lcd.setCursor(5,0);
  lcd.print(F("current="));
  lcd.print(TempoPrev);
  lcd.print(F(" "));
  lcd.setCursor(0,1);
  lcd.print(F("pot="));
  lcd.print(TempoPot);
  lcd.print(F(" "));
  lcd.setCursor(9,1);
  lcd.print(F("btn="));
  lcd.print(Tempo);
  lcd.print(F(" "));

  if (button1State == 1 && button1StatePrev == 0) { //PLAY
    button1StatePrev=1; 
    uClock.start();
    lcd.setCursor(0,0);
    lcd.print(F("PLAY "));
  }
  if (button1State == 0 && button1StatePrev == 1)  button1StatePrev=0;

  if (button2State == 1 && button2StatePrev == 0) {
    button2StatePrev=1; 
    uClock.stop();
    lcd.setCursor(0,0);
    lcd.print(F("STOP "));
  }
  if (button2State == 0 && button2StatePrev == 1) button2StatePrev=0;
  
/*
  if (button3State == 1 && button3StatePrev == 0) {
    button3StatePrev=1; 
    uClock.setTempo(Tempo);
  }
  if (button3State == 0 && button3StatePrev == 1) button3StatePrev=0;
*/
  if (button3State == 1) {
    if (TempoPrev != Tempo) {
      uClock.setTempo(Tempo);
      TempoPrev=Tempo;
    }
  }
  if (button4State == 1 && button4StatePrev == 0 && button5State == 0)  {
    Tempo--;
    button4StatePrev=1;
  }

  if (button5State == 1 && button5StatePrev == 0 && button4State == 0) {
    Tempo++;
    button5StatePrev=1;
  }

  if (button4State == 1 && button4StatePrev == 0 && button5State == 1)  {
    Tempo=Tempo + 10; 
    button4StatePrev=1;
  }

  if (button5State == 1 && button5StatePrev == 0 && button4State == 1) {
    Tempo=Tempo - 10; 
    button5StatePrev=1;
  }
  if (button4State == 0 && button4StatePrev == 1) button4StatePrev=0;
  if (button5State == 0 && button5StatePrev == 1) button5StatePrev=0;

  
  if (Tempo < 20) Tempo=20;
  if (Tempo > 275) Tempo=275;

 

}
